package com.swiveltechnologies.pinsafe.client.admin;

import java.util.Arrays;
import java.util.Date;
import java.util.EnumSet;
import java.util.Set;

/**
 * Simple Admin Client.
 */
public class SamplePINsafeAdminClient {

	/**
	 * PINsafe URL.
	 */
	private static final String PINSAFE = "http://localhost:8080/pinsafe";

	/**
	 * Some sample Admin requests.
	 */

	private static void addUser(String username) {
		UserAdminRequest r = new UserAdminRequest(PINSAFE, "secret");

		PINsafeUser user = new PINsafeUser(username);
		r.createUser(user);
		Set<PINsafeUserRight> rights = EnumSet.of(PINsafeUserRight.single, PINsafeUserRight.dual);
		setRights(user, rights, true);
		send(r);
	}

	private static void send(BaseAdminXmlRequest r) {
		boolean success;
		success = r.send();
		System.out.println((success ? "SUCCESS" : "FAILURE") + " " + r.getError());
		System.out.println(r.getResponse());
	}

	private static void adminRequests() {
		//
		UserAdminRequest r = new UserAdminRequest(PINSAFE, "secret");
		PINsafeUser ann, jim;
		boolean success;

		// User "ann"
		ann = new PINsafeUser("ann1");
		ann.attach(PINsafeUserPolicy.disabled, PINsafeUserPolicy.pinNeverExpires);
		ann.grant(PINsafeUserRight.single, PINsafeUserRight.dual);

		ann.setPIN("1234");

		// User "jim"
		jim = new PINsafeUser("jim1");
		jim.grant(PINsafeUserRight.single, PINsafeUserRight.helpdesk);
		jim.setPIN("5678");

		// Create both users and read ann
		r.createUsers(Arrays.asList(new PINsafeUser[] { ann, jim }));

		// Send the request and output the results
		send(r);
	}

	/**
	 * Some sample helpdesk requests.
	 */
	private static void helpdeskRequests() {
		//
		HelpdeskRequest r = new HelpdeskRequest(PINSAFE, "secret");
		PINsafeUser ann;
		boolean success;

		// User "ann"
		ann = new PINsafeUser("ann1");

		ann.detach(PINsafeUserPolicy.disabled);

		// Update ann's policy
		r.updateUser(ann);

		// Send the request and output the results
		success = r.send();
		System.out.println((success ? "SUCCESS" : "FAILURE") + " " + r.getError());
		System.out.println(r.getResponse());
	}

	private static void lock(String username, String repo, boolean lock) {
		PINsafeUser user;
		user = new PINsafeUser(username);
		if (!lock) {
			user.detach(PINsafeUserPolicy.locked);
		} else {
			user.attach(PINsafeUserPolicy.locked);
		}
		HelpdeskRequest r = new HelpdeskRequest(PINSAFE, "secret");
		r.updateUserGivenRepository(user, repo);

		send(r);
	}

	private static void setTransport(String username, String transport, String destination, boolean alert) {
		PINsafeUser user;
		user = new PINsafeUser(username);
		if (alert) {
			user.setStringTransport(transport, destination);
		} else {
			user.setAlertTransport(transport, destination);

		}
		UserAdminRequest r = new UserAdminRequest(PINSAFE, "secret");
		r.updateUser(user);

		boolean success = r.send();
		System.out.println((success ? "SUCCESS" : "FAILURE") + " " + r.getError());
		System.out.println(r.getResponse());
	}

	private static void sendStrings(String username) {
		PINsafeUser user;
		user = new PINsafeUser(username);
		HelpdeskRequest r = new HelpdeskRequest(PINSAFE, "secret");

		r.sendStringsForUser(user);

		boolean success = r.send();
		System.out.println((success ? "SUCCESS" : "FAILURE") + " " + r.getError());
		System.out.println(r.getResponse());
	}

	private static void setRight(String username, PINsafeUserRight right, boolean allow) {
		PINsafeUser user = new PINsafeUser(username);
		if (allow)
			user.grant(right);
		else
			user.revoke(right);

		UserAdminRequest r = new UserAdminRequest(PINSAFE, "secret");
		r.updateUser(user);
		boolean success = r.send();
		System.out.println((success ? "SUCCESS" : "FAILURE") + " " + r.getError());
		System.out.println(r.getResponse());

	}

	private static void setRights(PINsafeUser user, Set<PINsafeUserRight> rights, boolean allow) {
		if (allow) {
			for (PINsafeUserRight right : rights) {
				user.grant(right);
			}
		} else {
			for (PINsafeUserRight right : rights) {
				user.revoke(right);
			}
		}

		UserAdminRequest r = new UserAdminRequest(PINSAFE, "secret");
		r.updateUser(user);
		boolean success = r.send();
		System.out.println((success ? "SUCCESS" : "FAILURE") + " " + r.getError());
		System.out.println(r.getResponse());

	}

	private static void deleteUser(String username) {
		PINsafeUser user = new PINsafeUser(username);

		UserAdminRequest r = new UserAdminRequest(PINSAFE, "secret");
		r.deleteUser(new PINsafeUser(username));
		boolean success = r.send();
		System.out.println((success ? "SUCCESS" : "FAILURE") + " " + r.getError());
		System.out.println(r.getResponse());

	}

	private static void setPolicy(String username, PINsafeUserPolicy policy, boolean allow) {
		PINsafeUser user = new PINsafeUser(username);
		if (allow)
			user.attach(policy);
		else
			user.detach(policy);

		HelpdeskRequest r = new HelpdeskRequest(PINSAFE, "secret");
		r.updateUser(user);
		boolean success = r.send();
		System.out.println((success ? "SUCCESS" : "FAILURE") + " " + r.getError());
		System.out.println(r.getResponse());

	}

	private static void readUser(String username) {

		UserAdminRequest r = new UserAdminRequest(PINSAFE, "secret");
		r.readUser(new PINsafeUser(username));
		boolean success = r.send();
		System.out.println((success ? "SUCCESS" : "FAILURE") + " " + r.getError());
		System.out.println(r.getResponse());

	}

	private static void sync(String repName) {
		UserAdminRequest r = new UserAdminRequest(PINSAFE, "secret");
		r.syncRepo(repName);
		send(r);

	}

	private static void reportLocked(String rep) {
		PINsafeReport p = new PINsafeReport();
		p.requestLocked();
		p.setRepository(rep);
		UserAdminRequest r = new UserAdminRequest(PINSAFE, "secret");
		r.report(p);
		send(r);

	}

	private static void reportDisabled(String rep) {
		PINsafeReport p = new PINsafeReport();
		p.requestDisabled();
		p.setRepository(rep);
		UserAdminRequest r = new UserAdminRequest(PINSAFE, "secret");
		r.report(p);
		send(r);

	}

	private static void reportIdle(Date date) {
		PINsafeReport p = new PINsafeReport();
		p.requestIdle(date);
		p.setRepository("*");
		UserAdminRequest r = new UserAdminRequest(PINSAFE, "secret");
		r.report(p);
		send(r);

	}

	/**
	 * Entry point.
	 * 
	 * @param args
	 *            Unused.
	 */
	public static void main(final String[] args) {
		//
		// adminRequests();
		// sync("xxxx");
		/*
		 * unlock("test87678", "localxml"); setTransport("ann", "AQL",
		 * "447817360285", false); setTransport("ann", "AQL", "447817360285",
		 * true); //sendStrings("ann"); setRight("ann",
		 * PINsafeUserRight.helpdesk, false);
		 * 
		 * setPolicy("ann", PINsafeUserPolicy.changePin, true); setPolicy("ann",
		 * PINsafeUserPolicy.disabled, true); setPolicy("jim",
		 * PINsafeUserPolicy.locked, true); setPolicy("ann1",
		 * PINsafeUserPolicy.pinNeverExpires, true); readUser("ann1");
		 * deleteUser("ann");
		 */
/*		lock("test2", "xxxx", true);
		lock("jim1", "fred", true);
		reportLocked("xxxx");
		reportLocked("*");
		reportDisabled("*");
		reportLocked("fred");
		Date d = new Date(109, 05, 15);

		reportIdle(d);*/

		// helpdeskRequests();
		
		addUser("newuser");
	}

}
